/*******************************************************************************
 * Copyright (c) 2017, 2018 fortiss GmbH. 
 * 
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v2.0 which is available at
 * http://www.eclipse.org/legal/epl-v20.html
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Florian Hoelzl (fortiss GmbH) - initial implementation
 *
 *******************************************************************************/
package org.fortiss.tooling.base.ui.editor.fx.controller;

import static org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.DiagramViewerDefaultTags.LINK_TARGET_ALLOWED_TAG;
import static org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.DiagramViewerDefaultTags.LINK_TARGET_DENIED_TAG;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.DiagramCoordinate;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.controller.base.DelegatingContentAnchorageController;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.model.IModelChangeProvider;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.mvc.IContentAnchorageMVCBundle;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.mvc.IMVCBundle;
import org.eclipse.systemfocus.kernel.common.ui.javafx.lwfxef.mvc.MVCBundleTag;
import org.fortiss.tooling.base.model.element.IConnector;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.kernel.service.IConnectionCompositorService;
import org.fortiss.tooling.kernel.service.IElementCompositorService;

/**
 * {@link DelegatingContentAnchorageController} for
 * {@link org.fortiss.tooling.base.model.layout.ILayoutedModelElement}s.
 * 
 * @author hoelzl
 */
public class LayoutedModelElementBasedContentAnchorageController<T extends ILayoutedModelElement>
		extends DelegatingContentAnchorageController {
	/** {@link IModelChangeProvider} for this controller */
	private final LayoutModelElementModelChangeProvider layoutModelChangeProvider;

	/** Constructor. */
	public LayoutedModelElementBasedContentAnchorageController(IContentAnchorageMVCBundle mvcb,
			Class<T> modelType) {
		super(mvcb);

		Object model = mvcb.getModel();
		if(model == null) {
			throw new IllegalArgumentException("The given model is null!");
		}
		if(!modelType.isAssignableFrom(model.getClass())) {
			throw new IllegalArgumentException("Expected model of type " +
					modelType.getSimpleName() + ", but was " + model.getClass().getSimpleName());
		}

		ILayoutedModelElement lme = (ILayoutedModelElement)mvcb.getModel();
		this.layoutModelChangeProvider = new LayoutModelElementModelChangeProvider(lme);
	}

	/** Returns the model element. */
	public ILayoutedModelElement getLayoutedME() {
		return (ILayoutedModelElement)getModel();
	}

	/** {@inheritDoc} */
	@Override
	public IModelChangeProvider getModelChangeProvider() {
		return layoutModelChangeProvider;
	}

	/** {@inheritDoc} */
	@Override
	public void delete() {
		IElementCompositorService.getInstance().decompose(getLayoutedME());
	}

	/** {@inheritDoc} */
	@Override
	protected boolean canLink(IMVCBundle startBundle, DiagramCoordinate startLocation,
			IMVCBundle endBundle, DiagramCoordinate endLocation) {
		IConnectionCompositorService ccs = IConnectionCompositorService.getInstance();
		EObject start = (EObject)startBundle.getModel();
		EObject end = (EObject)endBundle.getModel();
		EObject root = (EObject)getViewer().getRootElement();
		return ccs.canConnect(start, end, root, null);
	}

	/** {@inheritDoc} */
	@Override
	protected void link(IMVCBundle startBundle, DiagramCoordinate startLocation,
			IMVCBundle endBundle, DiagramCoordinate endLocation) {
		EObject start = (EObject)startBundle.getModel();
		EObject end = (EObject)endBundle.getModel();
		EObject root = (EObject)getViewer().getRootElement();
		IConnectionCompositorService.getInstance().connect(start, end, root, null);
	}

	/** {@inheritDoc} */
	@Override
	public boolean isPossibleLinkTarget() {
		return true;
	}

	/** {@inheritDoc} */
	@Override
	public MVCBundleTag getLinkTargetEffectForNewLinkFrom(IMVCBundle linkSourceBundle) {
		Object srcModel = linkSourceBundle.getModel();
		if(srcModel instanceof ILayoutedModelElement && srcModel instanceof IConnector) {
			EObject startModel = (EObject)srcModel;
			ILayoutedModelElement endModel = getLayoutedME();
			EObject root = (EObject)getViewer().getRootElement();
			IConnectionCompositorService srv = IConnectionCompositorService.getInstance();
			if(srv.canConnect(startModel, endModel, root, null)) {
				return LINK_TARGET_ALLOWED_TAG;
			}
		}
		return LINK_TARGET_DENIED_TAG;
	}
}
